@@ -15,7 +15,7 @@ from contextlib import contextmanager
 from enum import Enum
 
 import zmq
-from traitlets import Any  # type: ignore
+from traitlets import Any
 from traitlets import Bool
 from traitlets import default
 from traitlets import DottedObjectName
@@ -25,7 +25,7 @@ from traitlets import observe
 from traitlets import observe_compat
 from traitlets import Type
 from traitlets import Unicode
-from traitlets.utils.importstring import import_item  # type: ignore
+from traitlets.utils.importstring import import_item
 
 from .connect import ConnectionFileMixin
 from .managerabc import KernelManagerABC
@@ -52,33 +52,40 @@ class _ShutdownStatus(Enum):
     SigkillRequest = "SigkillRequest"
 
 
-def in_pending_state(method):
-    """Sets the kernel to a pending state by
-    creating a fresh Future for the KernelManager's `ready`
-    attribute. Once the method is finished, set the Future's results.
-    """
+F = t.TypeVar('F', bound=t.Callable[..., t.Any])
 
-    @functools.wraps(method)
-    async def wrapper(self, *args, **kwargs):
-        # Create a future for the decorated method
-        try:
-            self._ready = Future()
-        except RuntimeError:
-            # No event loop running, use concurrent future
-            self._ready = CFuture()
-        try:
-            # call wrapped method, await, and set the result or exception.
-            out = await method(self, *args, **kwargs)
-            # Add a small sleep to ensure tests can capture the state before done
-            await asyncio.sleep(0.01)
-            self._ready.set_result(None)
-            return out
-        except Exception as e:
-            self._ready.set_exception(e)
-            self.log.exception(self._ready.exception())
-            raise e
 
-    return wrapper
+def in_pending_state(prefix: str = '') -> t.Callable[[F], F]:
+    def decorator(method: F) -> F:
+        """Sets the kernel to a pending state by
+        creating a fresh Future for the KernelManager's `ready`
+        attribute. Once the method is finished, set the Future's results.
+        """
+
+        @t.no_type_check
+        @functools.wraps(method)
+        async def wrapper(self, *args, **kwargs):
+            # Create a future for the decorated method
+            name = f"{prefix}_ready"
+            future = getattr(self, name)
+            if not future or future.done():
+                future = self._future_factory()
+                setattr(self, name, future)
+            try:
+                # call wrapped method, await, and set the result or exception.
+                out = await method(self, *args, **kwargs)
+                # Add a small sleep to ensure tests can capture the state before done
+                await asyncio.sleep(0.01)
+                future.set_result(None)
+                return out
+            except Exception as e:
+                future.set_exception(e)
+                self.log.exception(future.exception())
+                raise e
+
+        return t.cast(F, wrapper)
+
+    return decorator
 
 
 class KernelManager(ConnectionFileMixin):
@@ -87,23 +94,21 @@ class KernelManager(ConnectionFileMixin):
     This version starts kernels with Popen.
     """
 
+    _ready: t.Optional[t.Union[Future, CFuture]]
+    _shutdown_ready: t.Optional[CFuture]
+
     def __init__(self, *args, **kwargs):
         super().__init__(**kwargs)
         self._shutdown_status = _ShutdownStatus.Unset
-        # Create a place holder future.
-        try:
-            asyncio.get_running_loop()
-            self._ready = Future()
-        except RuntimeError:
-            # No event loop running, use concurrent future
-            self._ready = CFuture()
+        self._ready = None
+        self._shutdown_ready = None
 
     _created_context: Bool = Bool(False)
 
     # The PyZMQ Context to use for communication with the kernel.
     context: Instance = Instance(zmq.Context)
 
-    @default("context")
+    @default("context")  # type:ignore[misc]
     def _context_default(self) -> zmq.Context:
         self._created_context = True
         return zmq.Context()
@@ -114,11 +119,13 @@ class KernelManager(ConnectionFileMixin):
     )
     client_factory: Type = Type(klass="jupyter_client.KernelClient")
 
-    @default("client_factory")
+    _future_factory: t.Type[CFuture] = CFuture
+
+    @default("client_factory")  # type:ignore[misc]
     def _client_factory_default(self) -> Type:
         return import_item(self.client_class)
 
-    @observe("client_class")
+    @observe("client_class")  # type:ignore[misc]
     def _client_class_changed(self, change: t.Dict[str, DottedObjectName]) -> None:
         self.client_factory = import_item(str(change["new"]))
 
@@ -131,12 +138,12 @@ class KernelManager(ConnectionFileMixin):
 
     kernel_spec_manager: Instance = Instance(kernelspec.KernelSpecManager)
 
-    @default("kernel_spec_manager")
+    @default("kernel_spec_manager")  # type:ignore[misc]
     def _kernel_spec_manager_default(self) -> kernelspec.KernelSpecManager:
         return kernelspec.KernelSpecManager(data_dir=self.data_dir)
 
-    @observe("kernel_spec_manager")
-    @observe_compat
+    @observe("kernel_spec_manager")  # type:ignore[misc]
+    @observe_compat  # type:ignore[misc]
     def _kernel_spec_manager_changed(self, change: t.Dict[str, Instance]) -> None:
         self._kernel_spec = None
 
@@ -156,7 +163,7 @@ class KernelManager(ConnectionFileMixin):
 
     kernel_name: Unicode = Unicode(kernelspec.NATIVE_KERNEL_NAME)
 
-    @observe("kernel_name")
+    @observe("kernel_name")  # type:ignore[misc]
     def _kernel_name_changed(self, change: t.Dict[str, Unicode]) -> None:
         self._kernel_spec = None
         if change["new"] == "python":
@@ -174,14 +181,25 @@ class KernelManager(ConnectionFileMixin):
         help="True if the MultiKernelManager should cache ports for this KernelManager instance"
     )
 
-    @default("cache_ports")
+    @default("cache_ports")  # type:ignore[misc]
     def _default_cache_ports(self) -> bool:
         return self.transport == "tcp"
 
     @property
-    def ready(self) -> Future:
-        """A future that resolves when the kernel process has started for the first time"""
-        return self._ready
+    def ready(self) -> CFuture:
+        """A future that resolves when the kernel process has started."""
+        if not self._ready:
+            self._ready = self._future_factory()
+        assert self._ready is not None
+        return self._ready  # type:ignore[return-value]
+
+    @property
+    def shutdown_ready(self) -> CFuture:
+        """A future that resolves when the kernel process has shut down."""
+        if not self._shutdown_ready:
+            self._shutdown_ready = self._future_factory()
+        assert self._shutdown_ready is not None
+        return self._shutdown_ready
 
     @property
     def ipykernel(self) -> bool:
@@ -229,7 +247,7 @@ class KernelManager(ConnectionFileMixin):
     # create a Client connected to our Kernel
     # --------------------------------------------------------------------------
 
-    def client(self, **kwargs) -> KernelClient:
+    def client(self, **kwargs: t.Any) -> KernelClient:
         """Create a client configured to connect to our kernel"""
         kw = {}
         kw.update(self.get_connection_info(session=True))
@@ -290,7 +308,7 @@ class KernelManager(ConnectionFileMixin):
 
         return [pat.sub(from_ns, arg) for arg in cmd]
 
-    async def _async_launch_kernel(self, kernel_cmd: t.List[str], **kw) -> None:
+    async def _async_launch_kernel(self, kernel_cmd: t.List[str], **kw: t.Any) -> None:
         """actually launch the kernel
 
         override in a subclass to launch kernel subprocesses differently
@@ -318,7 +336,9 @@ class KernelManager(ConnectionFileMixin):
         self._control_socket.close()
         self._control_socket = None
 
-    async def _async_pre_start_kernel(self, **kw) -> t.Tuple[t.List[str], t.Dict[str, t.Any]]:
+    async def _async_pre_start_kernel(
+        self, **kw: t.Any
+    ) -> t.Tuple[t.List[str], t.Dict[str, t.Any]]:
         """Prepares a kernel for startup in a separate process.
 
         If random ports (port=0) are being used, this method must be called
@@ -346,7 +366,7 @@ class KernelManager(ConnectionFileMixin):
 
     pre_start_kernel = run_sync(_async_pre_start_kernel)
 
-    async def _async_post_start_kernel(self, **kw) -> None:
+    async def _async_post_start_kernel(self, **kw: t.Any) -> None:
         """Performs any post startup tasks relative to the kernel.
 
         Parameters
@@ -361,8 +381,8 @@ class KernelManager(ConnectionFileMixin):
 
     post_start_kernel = run_sync(_async_post_start_kernel)
 
-    @in_pending_state
-    async def _async_start_kernel(self, **kw):
+    @in_pending_state()
+    async def _async_start_kernel(self, **kw: t.Any) -> None:
         """Starts a kernel on this host in a separate process.
 
         If random ports (port=0) are being used, this method must be called
@@ -454,8 +474,8 @@ class KernelManager(ConnectionFileMixin):
 
     cleanup_resources = run_sync(_async_cleanup_resources)
 
-    @in_pending_state
-    async def _async_shutdown_kernel(self, now: bool = False, restart: bool = False):
+    @in_pending_state('_shutdown')
+    async def _async_shutdown_kernel(self, now: bool = False, restart: bool = False) -> None:
         """Attempts to stop the kernel process cleanly.
 
         This attempts to shutdown the kernels cleanly by:
@@ -473,6 +493,8 @@ class KernelManager(ConnectionFileMixin):
             Will this kernel be restarted after it is shutdown. When this
             is True, connection files will not be cleaned up.
         """
+        # Reset the start ready future.
+        self._ready = self._future_factory()
         self.shutting_down = True  # Used by restarter to prevent race condition
         # Stop monitoring for restarting while we shutdown.
         self.stop_restarter()
@@ -493,7 +515,9 @@ class KernelManager(ConnectionFileMixin):
 
     shutdown_kernel = run_sync(_async_shutdown_kernel)
 
-    async def _async_restart_kernel(self, now: bool = False, newports: bool = False, **kw) -> None:
+    async def _async_restart_kernel(
+        self, now: bool = False, newports: bool = False, **kw: t.Any
+    ) -> None:
         """Restarts a kernel with the arguments that were used to launch it.
 
         Parameters
@@ -519,7 +543,7 @@ class KernelManager(ConnectionFileMixin):
             kernel.
         """
         if self._launch_args is None:
-            raise RuntimeError("Cannot restart the kernel. " "No previous call to 'start_kernel'.")
+            raise RuntimeError("Cannot restart the kernel. No previous call to 'start_kernel'.")
 
         # Stop currently running kernel.
         await ensure_async(self.shutdown_kernel(now=now, restart=True))
@@ -633,6 +657,7 @@ class AsyncKernelManager(KernelManager):
     )
     client_factory: Type = Type(klass="jupyter_client.asynchronous.AsyncKernelClient")
 
+    _future_factory: t.Type[Future] = Future  # type:ignore[assignment]
     _launch_kernel = KernelManager._async_launch_kernel
     start_kernel = KernelManager._async_start_kernel
     pre_start_kernel = KernelManager._async_pre_start_kernel
@@ -653,7 +678,7 @@ KernelManagerABC.register(KernelManager)
 
 
 def start_new_kernel(
-    startup_timeout: float = 60, kernel_name: str = "python", **kwargs
+    startup_timeout: float = 60, kernel_name: str = "python", **kwargs: t.Any
 ) -> t.Tuple[KernelManager, KernelClient]:
     """Start a new kernel, and return its Manager and Client"""
     km = KernelManager(kernel_name=kernel_name)
@@ -671,7 +696,7 @@ def start_new_kernel(
 
 
 async def start_new_async_kernel(
-    startup_timeout: float = 60, kernel_name: str = "python", **kwargs
+    startup_timeout: float = 60, kernel_name: str = "python", **kwargs: t.Any
 ) -> t.Tuple[AsyncKernelManager, KernelClient]:
     """Start a new kernel, and return its Manager and Client"""
     km = AsyncKernelManager(kernel_name=kernel_name)
@@ -689,7 +714,7 @@ async def start_new_async_kernel(
 
 
 @contextmanager
-def run_kernel(**kwargs) -> t.Iterator[KernelClient]:
+def run_kernel(**kwargs: t.Any) -> t.Iterator[KernelClient]:
     """Context manager to create a kernel in a subprocess.
 
     The kernel is shut down when the context exits.
