@@ -45,7 +45,7 @@ def config(transport):
 
 def _install_kernel(name="signaltest", extra_env=None):
     if extra_env is None:
-        extra_env = dict()
+        extra_env = {}
     kernel_dir = pjoin(paths.jupyter_data_dir(), "kernels", name)
     os.makedirs(kernel_dir)
     with open(pjoin(kernel_dir, "kernel.json"), "w") as f:
@@ -112,13 +112,13 @@ def zmq_context():
 
 
 @pytest.fixture(params=[AsyncKernelManager, AsyncKMSubclass])
-def async_km(request, config):
+async def async_km(request, config):
     km = request.param(config=config)
     return km
 
 
 @pytest.fixture
-def async_km_subclass(config):
+async def async_km_subclass(config):
     km = AsyncKMSubclass(config=config)
     return km
 
@@ -194,6 +194,7 @@ class TestKernelManagerShutDownGracefully:
 class TestKernelManager:
     def test_lifecycle(self, km):
         km.start_kernel(stdout=PIPE, stderr=PIPE)
+        kc = km.client()
         assert km.is_alive()
         is_done = km.ready.done()
         assert is_done
@@ -201,6 +202,7 @@ class TestKernelManager:
         assert km.is_alive()
         km.interrupt_kernel()
         assert isinstance(km, KernelManager)
+        kc.stop_channels()
         km.shutdown_kernel(now=True)
         assert km.context.closed
 
@@ -449,11 +451,11 @@ class TestAsyncKernelManager:
         await async_km.start_kernel(stdout=PIPE, stderr=PIPE)
         is_alive = await async_km.is_alive()
         assert is_alive
-        is_ready = async_km.ready.done()
-        assert is_ready
+        await async_km.ready
         await async_km.restart_kernel(now=True)
         is_alive = await async_km.is_alive()
         assert is_alive
+        await async_km.ready
         await async_km.interrupt_kernel()
         assert isinstance(async_km, AsyncKernelManager)
         await async_km.shutdown_kernel(now=True)
